﻿/**
 * @fileOverview ポップアップを扱います。
 */

/**
 * ポップアップの要素を作成し、その情報を保持します。
 * @class
 * @param {element} source        ポップアップとして表示する要素
 * @param {Boolean} [useFade]     フェードアウトを有効にするかどうか
 * @param {Boolean} [hideOnHover] ポップアップ上でホバーしたときに非表示にするかどうか
 */
function PopupItem() {
    this.initialise.apply(this, arguments);
}
PopupItem.prototype = {
	/**  ポップアップの親要素
	 * @type element
	 */
	container:   null,
	/**  ポップアップとして表示する要素の直近の親要素
	 * @type element
	 */
	content:     null,
	/**  ポップアップとして表示する要素
	 * @type element
	 */
	source:      null,
	/**  フェードアウトを有効にするかどうか
	 * @type Boolean
	 */
	useFade:     false,
	/**  ポップアップ上でホバーしたときに非表示にするかどうか
	 * @type Boolean
	 */
	hideOnHover: false,
	/** コンストラクタから自動的に呼ばれ、初期化処理を行います。 */
	initialise: function(source, useFade, hideOnHover) {
		var divPopupContainer          = document.createElement("div");
		divPopupContainer.className    = "popupContainer";
		var divShadowTopRight          = document.createElement("div");
		divShadowTopRight.className    = "shadowTopRight";
		var divShadowBottomLeft        = document.createElement("div");
		divShadowBottomLeft.className  = "shadowBottomLeft";
		var divShadowBottomRight       = document.createElement("div");
		divShadowBottomRight.className = "shadowBottomRight";
		var divPopupBorder             = document.createElement("div");
		divPopupBorder.className       = "popupBorder";
		var divShadowRight             = document.createElement("div");
		divShadowRight.className       = "shadowRight";
		var divShadowBottom            = document.createElement("div");
		divShadowBottom.className      = "shadowBottom";
		var divPopupBody               = document.createElement("div");
		divPopupBody.className         = "popupBody";

		divShadowBottom.appendChild(divPopupBody);
		divShadowRight.appendChild(divShadowBottom);
		divPopupBorder.appendChild(divShadowRight);
		divShadowBottomRight.appendChild(divPopupBorder);
		divShadowBottomLeft.appendChild(divShadowBottomRight);
		divShadowTopRight.appendChild(divShadowBottomLeft);
		divPopupContainer.appendChild(divShadowTopRight);

		this.source      = source;
		this.container   = divPopupContainer;
		this.content     = divPopupBody;
		this.useFade     = useFade;
		this.hideOnHover = hideOnHover;
    }
};


/**
 * 複数のポップアップをまとめて管理します。
 * @static
 */
var Popup = {
	/** PopupItem を格納した配列
	 * @type Array
	 */
	items:       new Array(),
	/** すでに初期化されたかどうかを表すフラグ
	 * @type Boolean
	 */
	initialised: false,
	f: null,
	/** 要素の位置とサイズを取得します。
	 * @param  {element} src 要素
	 * @return {object} left, top, width, height, right, bottom の各メンバから成るオブジェクト
	 */
	getRect: function(src) {
		var parent = src;
		var x = 0;
		var y = 0;
		while (parent) {
			x += parent.offsetLeft;
			y += parent.offsetTop;
			parent = parent.offsetParent;
		}
		if (src.parentNode.tagName == "LABEL") y += window.scrollY - 
			src.parentNode.parentNode.parentNode.scrollTop; // set y to the right position on the analyse dialog
		else if (src.parentNode.parentNode.className == "resPopup") y -= src.parentNode.parentNode.parentNode.scrollTop;
		return {left: x, top: y, width: src.offsetWidth, height: src.offsetHeight,
		        right: x + src.offsetWidth, bottom: y + src.offsetHeight};
	},
	/** 指定された座標が要素の領域内にあるかどうかを調べます。
	 * @param  {Number}  x x 座標（スクリーン座標）
	 * @param  {Number}  y y 座標（スクリーン座標）
	 * @param  {element} element 要素
	 * @return {Boolean} 座標が要素の領域内にあれば true
	 */
	isPtInElement: function(x, y, element) {
		var rc = this.getRect(element);
		//return (x >= rc.left) && (x <= rc.right) && (y >= rc.top) && (y <= rc.bottom);
		return (x >= rc.left) && (x < rc.right) && (y >= rc.top) && (y < rc.bottom);
	},
	/** 要素がすでにポップアップとして存在しているかどうかを調べます。
	 * @param  {element} source 要素
	 * @return {Boolean} 要素がすでに存在していれば true
	 */
	sourceExists: function(source) {
		for (var i = 0; i < this.items.length; i++) {
			if (this.items[i].source == source) return true;
		}
		return false;
	},
	/** ポップアップのサイズに応じて、適切な位置に移動します。
	 * @param  {element} item 要素
	 */
	reposition: function(item) {
		var rectSource = this.getRect(item.source);
		item.container.style.visibility = "hidden";

		const POPUP_MAX_HEIGHT = 400;

		if (item.container.offsetHeight > POPUP_MAX_HEIGHT) {
			item.content.style.height = POPUP_MAX_HEIGHT;
			item.content.style.overflow = "auto";
		}

		var popupHeight = item.container.offsetHeight;
		var left = rectSource.left;
		var top = Alt_enableDefaultUpperPopup ? rectSource.top - popupHeight : rectSource.bottom;

		if (!Alt_enableDefaultUpperPopup) {
			// ポップアップの下端が画面からはみ出しているかチェック
			if (top + popupHeight > window.scrollY + window.innerHeight) {
				var oppositeY = (rectSource.top - popupHeight);
				if (oppositeY >= window.scrollY + Nodes.header.offsetHeight) top = oppositeY;
			}

		} else {
			// ポップアップの上端が画面からはみ出しているかチェック
			if (top < window.scrollY) {
				var oppositeY = (rectSource.bottom + popupHeight);
				if (oppositeY <= window.scrollY + window.innerHeight) top = rectSource.bottom;
			}
		}

		if (item.container.offsetWidth > document.body.clientWidth) {
			left = 0;
		} else {
			if (rectSource.left + item.container.offsetWidth > window.scrollX + document.body.clientWidth)
				left = document.body.clientWidth - item.container.offsetWidth - window.scrollX;
		}
		item.container.style.left = left;
		item.container.style.top  = top;
		item.container.style.visibility = "visible";
//		item.container.style.position = "fixed";
	},
	/** ポップアップとして要素を追加、表示します。
	 * @param  {element} content      表示する要素
	 * @param  {element} source       トリガー元の要素（位置の調整に使用される）
	 * @param  {Boolean} hideOnHover  ポップアップ上でホバーしたときに非表示にするかどうか
	 */
	add: function(content, source, hideOnHover) {
		if (this.sourceExists(source)) return false;
		//var item = new PopupItem(source);
		var item = new PopupItem(source, SkinPref.getBool("enablePopupFade", Alt_enablePopupFade), hideOnHover);
		this.items.push(item);
		item.content.appendChild(content);
		document.getElementById("popupdummy").appendChild(item.container);
		this.reposition(item);
		
		if (!this.initialised) {
			this.f = this.remove.bind(this);
			window.addEventListener("mouseout",  this.f, false);
			window.addEventListener("mousemove", this.f, false);
			this.initialised = true;
		}

		return item;
	},
	/** ポップアップを削除します。
	 * @param  {event} e イベント
	 */
	remove: function(e) {
		if (ResNumberContextMenu.is_open) return;
		var x = e.pageX;
		var y = e.pageY;
		for (var i = this.items.length - 1; i >= 0; i--) {
			if (this.isPtInElement(x, y, this.items[i].source))    return;
			if (!this.items[i].hideOnHover) if (this.isPtInElement(x, y, this.items[i].container)) return;
			if (x < 10) return; //右クリメニューに乗った時用。たぶん本文の要素は最小で50くらいまでだと思う。
			//if (SkinPref.getBool("enablePopupFade", Alt_enablePopupFade)) {
			if (this.items[i].useFade) {
				this.removeChildWithFade(this.items[i].container);
			} else {
				document.getElementById("popupdummy").removeChild(this.items[i].container);
			}
			this.items.pop();
		}
		if (this.items.length == 0) {
			window.removeEventListener("mouseout",  this.f, false);
			window.removeEventListener("mousemove", this.f, false);
			this.initialised = false;
		}
	},
	remove_last: function() {
		var i = this.items.length - 1;
		if (i >= 0) {
			if (this.items[i].useFade) {
				this.removeChildWithFade(this.items[i].container);
			} else {
				document.getElementById("popupdummy").removeChild(this.items[i].container);
			}
			this.items.pop();
			if (this.items.length == 0) {
				window.removeEventListener("mouseout",  this.f, false);
				window.removeEventListener("mousemove", this.f, false);
				this.initialised = false;
			}
		}
	},
	/** 指定された要素が載っているポップアップを探します。
	 * @param  {element} content 要素
	 * @return {PopupItem} マッチした PopupItem
	 */
	findPopup: function(content) {
		for (var i = 0, n = this.items.length; i < n; i++) {
			if (this.items[i].content.firstChild == content) return this.items[i];
		}
		return null;
	},
	/** ポップアップのフェードアウト処理を行います。removeChildWithFade() から呼ばれます。
	 * @param  {String} id 要素の ID
	 */
	fadeOutProc: function(id) {
		var node = document.getElementById(id);
		if (node) {
			var fadeStep = SkinPref.getInt("valuePopupFadeStep", Alt_valuePopupFadeStep);
			switch(fadeStep){
			case 0:
				fadeStep = 0.1;  break;
			case 1:
				fadeStep = 0.2;  break;
			case 2:
				fadeStep = 0.25; break;
			case 3:
				fadeStep = 0.33; break;
			case 4:
				fadeStep = 0.5;  break;
			default:
				fadeStep = 0.2;
			}
			node.style.opacity = parseFloat(node.style.opacity) - fadeStep;
			if (parseFloat(node.style.opacity) <= 0.0) {
				document.getElementById("popupdummy").removeChild(node);
			} else {
				window.setTimeout(this.fadeOutProc.bind(this), 1, id);
			}
		}
	},
	/** ポップアップをフェードアウトしながら削除します。
	 * @param  {element} node 要素
	 */
	removeChildWithFade: function(node){
		var id = "#" + new Date().getTime() + Math.random();
		node.id = id
		if (!node.style.opacity) {
			node.style.opacity = 1.0;
		}
		window.setTimeout(this.fadeOutProc.bind(this), 1, id);
	}
};

// ResPopup オブジェクト
// レス番ポップアップ（と名前欄ポップアップ）
//
/**
 * レスアンカーと名前欄のポップアップを管理します。
 * @static
 */
var ResPopup = {
	popup_limit: null,
	/** イベントリスナを登録します。*/
	initialise: function() {
//tn
		window.addEventListener("mouseover", this.onMouseOver.bind(this), false);
		this.popup_limit = (SkinPref.getInt("valuePopupResMax", Alt_valuePopupResMax) + 1) * 5;
	},
	/** 全角文字で表記された数字を、数値型に変換します。
	 * @param  {String} str 全角数字が含まれた文字列
	 * @return {Number} 変換された数値
	 */
	toNarrow: function(str) {
		return parseInt(str.replace(/([０-９])/g, function(n){ return String.fromCharCode(n.charCodeAt(0) - 0xFEE0); }));
	},
	/** 要素を複製し、いくつかの属性を削除したものを返却します。
	 * @param  {element} src 要素
	 * @return {element} 複製された要素
	 */
	getCloneNode: function(src){
		var dest = src.cloneNode(true);
		if (dest.id) dest.id = "";
		dest.removeAttribute("name");
		dest.style.display = "block";
		var e = dest.getElementsByTagName("*");
		for(var i=0; i<e.length; i++){
			if(e[i].id) e[i].id = "";
			e[i].removeAttribute("name");
		}
		return dest;
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		switch (node.className) {
			case "resPointer":
				if (node.textContent.match(/\,/)) {
					var min = 1000;
					var max = 1;
					var expAnchor = new RegExp(/([0-9０-９]{1,4})[\-－]?([0-9０-９]{0,4})/);
					var used = {};
					var rest = this.popup_limit;

					var str_array = node.textContent.split(",");
					L1: for (var i = 0, n = str_array.length; i < n; i++)
						if (str_array[i].match(expAnchor)) {
							var start = RegExp.$1;
							var end   = RegExp.$2;
							start = this.toNarrow(start);
							end = end ? this.toNarrow(end) : start;
							if (start > 1000) break;
							for (var j = start; j <= end; j++) {
								used[j] = 1;
								min = (min > start ? start : min);
								max = (max < end ? end : max);
								rest--;
								if (rest <= 0) break L1;
							}
						}
					this.show(node, min, max, used);
				} else
				if (node.textContent.match(/(>|＞)([0-9０-９]{1,4})[\-－]([0-9０-９]{1,4})/)) {
					var s = RegExp.$2;
					var e = RegExp.$3;
					this.show(node, this.toNarrow(s), this.toNarrow(e));
				} else if (node.textContent.match(/(>|＞)([0-9０-９]{1,4})/)) {
					var s = this.toNarrow(RegExp.$2);
					this.show(node, s, s);
				}
				break;
			case "resName":
				node.style.cursor = "pointer";
				if (node.textContent.match(/[\,=]/)) {
					var min = 1000;
					var max = 1;
					var expAnchor = new RegExp(/^\s*([0-9０-９]{1,4})[\-－]?([0-9０-９]{0,4})/);
					var used = {};
					var rest = this.popup_limit;

					var str_array = node.textContent.split(/[\,=]/);
					L1: for (var i = 0, n = str_array.length; i < n; i++)
						if (str_array[i].match(expAnchor)) {
							var start = RegExp.$1;
							var end   = RegExp.$2;
							start = this.toNarrow(start);
							end = end ? this.toNarrow(end) : start;
							if (start > 1000) break;
							for (var j = start; j <= end; j++) {
								used[j] = 1;
								min = (min > start ? start : min);
								max = (max < end ? end : max);
								rest--;
								if (rest == 0) break L1;
							}
						}
					this.show(node, min, max, used);
				} else if (node.textContent.match(/^\s*([0-9０-９]{1,4})[\-－]([0-9０-９]{1,4})$/)) {
					var s = RegExp.$1;
					var e = RegExp.$2;
					this.show(node, this.toNarrow(s), this.toNarrow(e));
				} else if (node.textContent.match(/^\s*([0-9０-９]{1,4})(\s*◆)?/)) {
					var s = this.toNarrow(RegExp.$1);
					this.show(node, s, s);
				}
				node.setAttribute("onclick", "ThreadDocument.jumpTo(event)")
		}
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source トリガー元の要素
	 * @param {Number}  start  ポップアップを開始するレス番号
	 * @param {Number}  end    ポップアップを終了するレス番号
	 */
	show: function(source, start, end, items, func) {
		if (!SkinPref.getBool("enableResPopup", Alt_enableResPopup)) return;
		if (end < start) return;
		if (start < 1)  start = 1;
		if (end > 1000) end = 1000;
		var force = false;
		if (!items) {
			items = {"0" : 1};
			force = true;
			if ((end - start) > this.popup_limit) end = start + this.popup_limit - 1;
		}
		Trackback.traverse();
		var content = document.createDocumentFragment();
		for(var i = start; i <= end; i++){
			if (force || items[i]) {
			resNode = Nodes.getRes(i);
			if(!resNode) {
				this.showExceeded(source, start, end, items, func);
				return;
			}
			if (func) {
				if (!func(resNode)) continue;
			}
			var dlPopup = this.getCloneNode(resNode);
			dlPopup.className = "resPopup";
			Trackback.appendTo(dlPopup, i);
			content.appendChild(dlPopup);
			}
		}
		var popupItem = Popup.add(content, source);
		// error ...
		if (popupItem) ThreadDocument.modifyContent(popupItem.content, true, true);
	},
	/** 表示域外のポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source トリガー元の要素
	 * @param {Number}  start  ポップアップを開始するレス番号
	 * @param {Number}  end    ポップアップを終了するレス番号
	 */
	showExceeded: function(source, start, end, items, func) {
		var xmlHTTP = new XMLHttpRequest();
		xmlHTTP.onreadystatechange = xmlEventHandler.bind(this);
		function xmlEventHandler() {
			if (xmlHTTP.readyState == 4) {
				if (xmlHTTP.status == 200) {
					var content = document.createDocumentFragment();
					var divTemp  = document.createElement("div");
					divTemp.innerHTML = xmlHTTP.responseText;
					var dl = divTemp.getElementsByTagName("dl");
					var force = (items["0"] ? true : false);
					for (var i = 0, n = dl.length; i < n; i++) {
						if (force || items[start + i]) {
							if (func) {
								if (!func(dl[i])) continue;
							}
							var dlPopup = this.getCloneNode(dl[i]);
							dlPopup.className = "resPopup";
							Trackback.appendTo(dlPopup, start + i);
							content.appendChild(dlPopup);
						}
					}
					var popupItem = Popup.add(content, source);
					if (popupItem) ThreadDocument.modifyContent(popupItem.content, true, true);
				}
			}
		}
		xmlHTTP.open("GET", SERVER_URL + EXACT_URL + start + "-" + end + "n" + "?lite=true", true);
		xmlHTTP.send(null);
	}
};

/**
 * ID のポップアップを管理します。
 * @static
 */
var IDPopup = {
	/** イベントリスナを登録します。*/
	initialise: function() {
		if (SkinPref.getBool("enableIDPopupOnClick", Alt_enableIDPopupOnClick)) {
			window.addEventListener("click", this.onMouseOver.bind(this), false);
		} else {
			window.addEventListener("mouseover", this.onMouseOver.bind(this), false);
		}
	},
	/** 要素を複製し、いくつかの属性を削除したものを返却します。
	 * @param  {element} src 要素
	 * @return {element} 複製された要素
	 */
	getCloneNode: function(src){
		var dest = src.cloneNode(true);
		if (dest.id) dest.id = "";
		dest.removeAttribute("name");
		dest.style.display = "block";
		var e = dest.getElementsByTagName("*");
		for(var i=0; i<e.length; i++){
			if(e[i].id) e[i].id = "";
			e[i].removeAttribute("name");
		}
		return dest;
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		if (e.button != 0) return;
		var node = e.target;
		if (node.tagName == "SPAN") { // avoid conflict between Moreyon
			if (node.className.match(/mesID_/)) {
				var pos = node.className.indexOf(" ");
				var className = (pos == -1) ? node.className : node.className.slice(0, pos);
				IDPopup.show(node, className.slice(6), true);
			} else if (node.className.match(/^resID/)) {
				IDPopup.show(node, node.getAttribute("rel"), false);
			}
		}
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source      トリガー元の要素
	 * @param {Number}  id          ポップアップする ID
	 * @param {bool}    popupOnBody ポップアップを終了するレス番号
	 */
	show: function(source, id, popupOnBody) {
		if (!SkinPref.getBool("enableIDPopup", Alt_enableIDPopup)) return;
		if (id.length < 8) return;
		const IDENTIFIER = "IDPopup:";
		if (document.getElementById(IDENTIFIER + id)) return;
		ID.traverse();
		var items = ID.items[id];
		if (!items) return;
		if (items.length < (popupOnBody ? 1 : 2)) return;
		
		var appendAll = SkinPref.getBool("enableIDPopupAll", Alt_enableIDPopupAll);
		var content = document.createDocumentFragment();
		var div = document.createElement("div");
		div.id = IDENTIFIER + id;
		div.className = "popupHeader";
		if (!appendAll) {
			div.appendChild(document.createTextNode("参照: "));
			content.appendChild(div);
		}

		var span = document.createElement("span");
		var br = document.createElement("br");
		var div_num = SkinPref.getInt("valueTrackBackDivNums", Alt_valueTrackBackDivNums);
		var a = document.createElement("a");
		a.className = "resPointer";
		a.href = "javascript:void(0)";
		a.onclick = ThreadDocument.jumpTo.bind(ThreadDocument);

//		var pickup = SkinPref.getBool("enableIDPickup", Alt_enableIDPickup);

		for (var i = 0, nums = items.length; i < nums; i++) {
			if (appendAll) {
				var dlPopup = this.getCloneNode(Nodes.getRes(items[i]));
				dlPopup.className = "resPopup";
				Trackback.appendTo(dlPopup, items[i]);
				content.appendChild(dlPopup);
			}
			var anchor = a.cloneNode(false);
			anchor.appendChild(document.createTextNode(">>" + items[i]));
			div.appendChild(anchor);
			if (i != nums - 1) {
				var space = span.cloneNode(false);
				space.appendChild(document.createTextNode(" "));
				div.appendChild(space);
				if ((i + 1) % div_num == 0) {
					var newline = br.cloneNode(false);
					div.appendChild(newline);
				}
			}
//			if (pickup) {
//				var id = Nodes.getHeader(items[i]).id;
//				Nodes.getHeader(items[i]).id = (id ? "" : "pickup");
//			}
		}
		
		var nodePopup = Popup.add(content, source);
		if (!appendAll) nodePopup.container.className = "popupResList";
		if (nodePopup) ThreadDocument.modifyContent(nodePopup.content, true, true);
	}
};

/**
 * 逆参照のポップアップを管理します。
 * @static
 */
var TrackbackPopup = {
	/** 要素を複製し、いくつかの属性を削除したものを返却します。
	 * @param  {element} src 要素
	 * @return {element} 複製された要素
	 */
	getCloneNode: function(src){
		var dest = src.cloneNode(true);
		if (dest.id) dest.id = "";
		dest.removeAttribute("name");
		dest.style.display = "block";
		var e = dest.getElementsByTagName("*");
		for(var i=0; i<e.length; i++){
			if(e[i].id) e[i].id = "";
			e[i].removeAttribute("name");
		}
		return dest;
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		if (node.textContent.match(/^([0-9０-９]{1,4})$/)) {
//ポップアップ内のレス番号ではトラックバックを呼ばない。(下に参照が出てるし)
			if(node.parentNode.parentNode.parentNode.className == "resPopup") return;
			this.show(node, RegExp.$1);
		}
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source      トリガー元の要素
	 * @param {Number}  number      ポップアップするレス番号
	 */
	show: function(source, number) {
		if (!SkinPref.getBool("enableTrackBackPopup", Alt_enableTrackBackPopup)) return;
		Trackback.traverse();
		var tb = Trackback.items[number];
		if (tb.length < 1) return;
		var appendAll = SkinPref.getBool("enableTrackBackPopupAll", Alt_enableTrackBackPopupAll) || tb.length == 1;
		var content = document.createDocumentFragment();
		if (appendAll) {
			for (var i = 0, n = tb.length; i < n; i++) {
				var node = this.getCloneNode(ResNodes.getContainerByIndex(tb[i]));
				node.className = "resPopup";
				Trackback.appendTo(node, tb[i]);
				content.appendChild(node);
			}
		} else {
			var div = document.createElement("div");
			div.className = "popupHeader";
			div.appendChild(document.createTextNode("参照: "));
			div.appendChild(Trackback.getAnchorElements(number));
			content.appendChild(div);
		}
		var nodePopup = Popup.add(content, source);
		if (!appendAll) nodePopup.container.className = "popupResList";
		if (nodePopup) ThreadDocument.modifyContent(nodePopup.content, true, true);
	}
};

/**
 * 画像のポップアップを管理します。
 * @static
 */
var ImagePopup = {
	/** 画像の URI を判別する正規表現
	 * @type RegExp */
	regExp: /\.(jpg|jpeg|png|gif|mng|tiff|tif|bmp|pict)$/i,
	/** 危険なレスかどうかを判別する正規表現
	 * @type RegExp */
	grotesqueRegExp: /(ｸﾞﾛ|グロ|注意|危険|有害|ブラクラ|ﾌﾞﾗｸﾗ|精神|蓮|氏ね|死|血)/,
	/** 対象の URI が画像かどうかを判別します。
	 * @param  {String}  src URI
	 * @return {Boolean} 画像であれば true
	 */
	isImage: function(src) {
		return this.regExp.test(src) || /http:\/\/imepita\.jp\/\d{8}\/\d{6}/.test(src);
	},
	/** 対象のレス番号にグロテスク画像への注意を喚起するレスがついているか調べます。
	 * @param  {Number} index レス番号
	 * @return {Boolean} 危険なレスであれば true
	 */
	isImageGrotesque: function(index) {
		Trackback.traverse();
//		var nodes = Trackback.items[index];
		// 直後のレスもチェック対象に含める
		var nodes = Array.apply(null, Trackback.items[index]);
		nodes.unshift(index + 1);
		if (nodes) {
			for (var i = 0, n = nodes.length; i < n; i++) {
				var node = ResNodes.getBodyByIndex(nodes[i]);
				if (node) {
					if (node.textContent.match(this.grotesqueRegExp)) {
						return true;
					}
				}
			}
		}
		return false;
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		//var src = node.href;
		var src = node.rel;
		//if (src.match(/\?url=/)) src = RegExp.rightContext;
		if (this.isImage(src)) {
			if (/http:\/\/imepita\.jp\/\d{8}\/\d{6}/.test(src)) {
				src = '/imepita/' + src;
			}
			var is_gro = this.isImageGrotesque(ResNodes.getIndexByBody(node.parentNode));
			if (e.ctrlKey ^ SkinPref.getBool("enableDefaultPixelation", Alt_enableDefaultPixelation) || is_gro) {
				var gro_level = is_gro ^ e.ctrlKey;
				if (SkinPref.getInt("valuePixelationMethod", Alt_valuePixelationMethod) == 0) {
					this.pixelateShow(node, src, gro_level);
				} else {
					this.blurShow(node, src, gro_level);
				}
			} else {
				this.show(node, src);
			}
			if (is_gro)
				node.setAttribute("title", "表示注意");
		}
	},
	/** click イベントを処理します。
	 * @param  {event} e イベント
	 */
	onClick: function(e) {
		var node  = e.currentTarget;
		var popup = Popup.findPopup(node);
		if (popup) {
			if (node.className == "imgPopup") {
				node.className =  "imgLargePopup";
				popup.content.style.maxWidth  = "none";
				popup.content.style.maxHeight = "none";
				popup.content.style.maxWidth  = e.currentTarget.offsetWidth  - 7;
				popup.content.style.maxHeight = e.currentTarget.offsetHeight - 7;
//popup.content.style.height = 500;
//popup.content.style.overflow = "auto";
			} else {
				if (node.tagName.match(/(canvas|svg)/i)) {
					popup.container.style.opacity = 1;
					node.parentNode.childNodes[1].style.opacity = 1;
					node.parentNode.removeChild(node);
				} else {
					e.currentTarget.className =  "imgPopup";
//popup.content.style.overflow = "visible";
					popup.content.style.maxWidth  = e.currentTarget.offsetWidth  - 7;
					popup.content.style.maxHeight = e.currentTarget.offsetHeight - 7;
				}
			}
		}
	},
	/** error イベントを処理します。
	 * @param  {event} e イベント
	 */
	onError: function(e) {
		//var popup = Popup.findPopup(e.currentTarget.parentNode);
		var popup = Popup.findPopup(e.currentTarget);
		if (popup) {
			popup.source.setAttribute("title", popup.source.title ? "エラー " + popup.source.title : "エラー");
		}
		e.currentTarget.parentNode.removeChild(e.currentTarget);
	},
	/** 画像のリサイズを監視します。show() から呼ばれます。画像のサイズが確定すると、ポップアップのサイズを調整し実際に表示します。
	 * @param  {PopupItem} nodePopup PopupItem オブジェクト
	 */
	resize: function(nodePopup) {
		if (!nodePopup) return;
		if (!nodePopup.content.lastChild) return;
		if ((nodePopup.content.lastChild.offsetWidth != 0) && (nodePopup.content.lastChild.offsetHeight != 0)) {
			if ((nodePopup.content.lastChild.offsetWidth != 24) && (nodePopup.content.lastChild.offsetHeight != 24)) {
			//if ((nodePopup.content.firstChild.offsetWidth != 28) && (nodePopup.content.firstChild.offsetHeight != 28)) {
				//nodePopup.container.className = "popupImage";
				//nodePopup.content.firstChild.className = "imgPopup";
				nodePopup.content.style.maxWidth  = nodePopup.content.lastChild.offsetWidth  - 7;
				nodePopup.content.style.maxHeight = nodePopup.content.lastChild.offsetHeight - 7;
				Popup.reposition(nodePopup);
				nodePopup.container.style.visibility = "visible";
//ぼかしレベル。(オプションから指定)
				switch (SkinPref.getInt("valueShadeLevel", Alt_valueShadeLevel)) {
				case 0:
					nodePopup.container.style.opacity = 0.15; break;
				case 1:
					nodePopup.container.style.opacity = 0.3;  break;
				case 2:
					nodePopup.container.style.opacity = 0.5;  break;
				case 3:
					nodePopup.container.style.opacity = 1;    break;
				default:
					nodePopup.container.style.opacity = 0.5;  break;
				}
				return;
			}
		}
		window.setTimeout(this.resize.bind(this), 50, nodePopup);
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source   トリガー元の要素
	 * @param {String}  href      画像の URI
	 */
	show: function(source, href) {
		if (!SkinPref.getBool("enableImagePopup", Alt_enableImagePopup)) return;
		source.removeAttribute("title");
		var img = document.createElement("img");
		img.onerror = this.onError.bind(this);
		img.onclick = this.onClick.bind(this);
		var nodePopup = Popup.add(img, source);
		if (nodePopup) {
			nodePopup.container.className = "popupImage";
			if (SkinPref.getBool("enableImagePopupShadeOverCancel", Alt_enableImagePopupShadeOverCancel)) nodePopup.container.className = "popupImageC";
			img.className = "imgPopup";
			nodePopup.container.style.visibility = "hidden";
			nodePopup.useFade = false;
			//alert("");
			img.src = href;
			window.setTimeout(this.resize.bind(this), 50, nodePopup);
		}
	},
	/** 画像をモザイク化してポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source   トリガー元の要素
	 * @param {String}  href      画像の URI
	 */
	pixelateShow: function(source, href, is_gro) {
		if (!SkinPref.getBool("enableImagePopup", Alt_enableImagePopup)) return;
		source.removeAttribute("title");
		var img = document.createElement("img");
		img.onerror = this.onError.bind(this);
		img.onclick = this.onClick.bind(this);
		img.style.opacity = 0.2;
		var nodePopup = Popup.add(img, source);
		var f = (function() {
			var canvas = document.createElement("canvas");
			canvas.onclick   = this.onClick.bind(this);
			canvas.style.position   = "absolute";
			canvas.style.zIndex = 1;
			canvas.width  = img.offsetWidth;
			canvas.height = img.offsetHeight;
			var pxsize = is_gro ? 0 : SkinPref.getInt("valuePixelationSize", Alt_valuePixelationSize);
			switch (pxsize) {
				case 0:
					var px = 8; break;
				case 1:
					var px = 4; break;
				case 2:
					var px = 2; break;
			}
			var ctx = canvas.getContext("2d");
			ctx.globalCompositeOperation = "copy";
			var w = canvas.width  / px;
			var h = canvas.height / px;
			ctx.drawImage(img, 0, 0, w, h);
			ctx.drawImage(canvas, 0, 0, w, h, 0, 0, canvas.width + px, canvas.height + px);
			if (SkinPref.getBool("enablePrecisePixelation", Alt_enablePrecisePixelation)) {
				var m = px / 2;
				for (var y = 0; y < canvas.height; y += px) {
					for (var x = 0; x < canvas.width; x += px) {
						ctx.drawImage(canvas, x + m, y + m, 1, 1, x, y, px, px);
					}
				}
			}
			nodePopup.content.insertBefore(canvas, nodePopup.content.firstChild);
		}).bind(this);
		if (nodePopup) {
			nodePopup.container.className = "popupImage";
			if (SkinPref.getBool("enableImagePopupShadeOverCancel", Alt_enableImagePopupShadeOverCancel)) nodePopup.container.className = "popupImageC";
			img.className = "imgPopup";
			nodePopup.container.style.visibility = "hidden";
			nodePopup.useFade = false;
			img.onload = f;
			window.setTimeout(this.resize.bind(this), 50, nodePopup);
			img.src = href;
		}
	},
	/** 画像にガウスぼかしをかけてポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source   トリガー元の要素
	 * @param {String}  href      画像の URI
	 */
	blurShow: function(source, href, is_gro) {
		if (!SkinPref.getBool("enableImagePopup", Alt_enableImagePopup)) return;
		source.removeAttribute("title");
		var img = document.createElement("img");
		img.style.opacity = 0.2;
		img.onerror = this.onError.bind(this);
		img.onclick = this.onClick.bind(this);
		var nodePopup = Popup.add(img, source);
		var f = (function() {
			var pxsize = is_gro ? 0 : SkinPref.getInt("valuePixelationSize", Alt_valuePixelationSize);
			switch (pxsize) {
				case 0:
					var px = 8; break;
				case 1:
					var px = 4; break;
				case 2:
					var px = 2; break;
			}
			var canvas = new SVGCanvas(img.offsetWidth, img.offsetHeight);
			canvas.svg.onclick = this.onClick.bind(this);
			canvas.svg.style.position  = "absolute";
			canvas.svg.style.zIndex = 1;
			var defs   = new SVGDefinitions();
			var filter = new SVGFilter("blur");
			var effect = new SVGFilterPrimitives.GaussianBlur(px);
			filter.add(effect);
			defs.add(filter);
			var layer = new SVGLayer();
			var element = layer.image(img.src, 0, 0, img.offsetWidth, img.offsetHeight);
			element.setAttribute("filter", "url(#blur)");
			canvas.add(defs);
			canvas.add(layer);
			nodePopup.content.insertBefore(canvas.svg, nodePopup.content.firstChild);
		}).bind(this);
		if (nodePopup) {
			nodePopup.container.className = "popupImage";
			if (SkinPref.getBool("enableImagePopupShadeOverCancel", Alt_enableImagePopupShadeOverCancel)) nodePopup.container.className = "popupImageC";
			img.className = "imgPopup";
			nodePopup.container.style.visibility = "hidden";
			nodePopup.useFade = false;
			img.onload = f;
			window.setTimeout(this.resize.bind(this), 50, nodePopup);
			img.src = href;
		}
	}
};

/**
 * リンク先のサムネイルプレビューのポップアップを管理します。
 * @static
 */
var UrlPopup = {
	/** URI がサムネイル画像へのリクエストを発行すべきか判別します。
	 * @param  {String} src URI
	 * @return {Boolean} リクエストすべきであれば true
	 */
	isValid: function(src) {
		if (src.match(/(http|https):\/\/.+\/.*\.([a-zA-Z0-9]+)$/)) {
			var ext = RegExp.$2;
			return ext.match(/^(htm|html|shtm|shtml|stm|xml|xhtml|php|php3|php4|cgi|jsp|cfm|asp|aspx|pl|plx|py|rb|)$/);
		} else {
			return true;
		}
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		//var src = node.href;
		var src = node.rel;
		//if (src.match(/\?url=/)) src = RegExp.rightContext;
		if (src.match(/2ch.net/)) return;
		if (src.match(/read\.cgi/)) return;
		if (ImagePopup.isImage(src)) return;
		if (VideoPopup.getVideoSource(src)) return;
		if (this.isValid(src)) this.show(node, src);
	},
	/** error イベントを処理します。
	 * @param  {event} e イベント
	 */
	onError: function(e) {
		var popup = Popup.findPopup(e.currentTarget.parentNode);
		if (popup) {
			popup.source.setAttribute("title", "エラー");
		}
		e.currentTarget.parentNode.removeChild(e.currentTarget);
	},
	/** 画像のリサイズを監視します。show() から呼ばれます。画像のサイズが確定すると、ポップアップのサイズを調整し実際に表示します。
	 * @param  {PopupItem} nodePopup PopupItem オブジェクト
	 */
//?
	resize: function(nodePopup) {
		if (!nodePopup) return;
		if (!nodePopup.content.firstChild.firstChild) return;
		if ((nodePopup.content.firstChild.firstChild.offsetWidth != 0) && (nodePopup.content.firstChild.firstChild.offsetHeight != 0)) {
			if ((nodePopup.content.firstChild.firstChild.offsetWidth != 28) && (nodePopup.content.firstChild.firstChild.offsetHeight != 28)) {
				nodePopup.container.className = "popupImage";
				nodePopup.content.firstChild.firstChild.className = "urlPopup";
				nodePopup.content.style.maxWidth  = nodePopup.content.firstChild.firstChild.offsetWidth  - 7;
				nodePopup.content.style.maxHeight = nodePopup.content.firstChild.firstChild.offsetHeight - 7;
				Popup.reposition(nodePopup);
				nodePopup.container.style.visibility = "visible";
				return;
			}
		}
		window.setTimeout(this.resize.bind(this), 10, nodePopup);
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source   トリガー元の要素
	 * @param {String}  href     リンク先の URI
	 */
	show: function(source, href) {
		if (!SkinPref.getBool("enableUrlPopup", Alt_enableUrlPopup)) return;
		source.removeAttribute("title");
		var a    = document.createElement("a");
		a.href   = href;
		if (SkinPref.getBool("enableNewWindow", Alt_enableNewWindow)) a.target = "_blank";
		var img = document.createElement("img");
		switch (SkinPref.getInt("valueThumbnailSite", Alt_valueThumbnailSite)) {
//			case 0:
//				img.src = "http://shots.snap.com/preview/?url=" + encodeURIComponent(href) + "&tok=000147da6bd8e7450d48b1588299a3b32e3d27c7&size=" +
//					(SkinPref.getInt("valueUrlPopupSize", Alt_valueUrlPopupSize) == 0 ? "large" : "small");
//				break;
			case 1:
				img.src = "http://img.simpleapi.net/small/" + href;
				break;
			case 2:
				img.src = "http://capture.heartrails.com/" +
					(SkinPref.getInt("valueUrlPopupSize", Alt_valueUrlPopupSize) == 0 ? "large" : "small") + "?" + href;
				break;
			case 3:
			default:
				img.src = "http://images.websnapr.com/?url=" + encodeURIComponent(href) + "&size=" +
					(SkinPref.getInt("valueUrlPopupSize", Alt_valueUrlPopupSize) == 0 ? "S" : "T");
				break;
		};

		img.setAttribute("onerror", "UrlPopup.onError(event)");
		a.appendChild(img);
		var nodePopup = Popup.add(a, source, true);
		if (nodePopup) {
			nodePopup.container.style.visibility = "hidden";
			window.setTimeout(this.resize.bind(this), 10, nodePopup);
		}
	}
};


/**
 * 動画サイトのポップアップを管理します。
 * @static
 */
var VideoPopup = {
	/** URI から動画のプレビューのための正しいソース URI を取得します。
	 * @param  {String} src URI
	 * @param  {Boolean} インライン表示フラグ
	 * @return {String} ソース URI
	 */
	getVideoSource: function(src, is_embed) {
		//if (src.match(/youtube\.com\/.*\?v=([^\&]+)/)) {
		if (src.match(/youtube\.com\/watch\?v=([^&]+)/)) {
			var option = "";
			if (!is_embed)
				if (SkinPref.getBool("videoPopupAutoStart", Alt_videoPopupAutoStart)) option = "&autoplay=1";
			if (SkinPref.getBool("enableVideoHighQuality", Alt_enableVideoHighQuality))
				option = option + "&ap=%2526fmt=18";
      		return "http://www.youtube.com/v/" + RegExp.$1 + option;
	    } else if (src.match(/video.google.*\?docid=([\d\-]+)/)) {
	    	return "http://video.google.com/googleplayer.swf?docId=" + RegExp.$1;
	    } else if (src.match(/metacafe.com\/watch\/(\d+)/)){
	    	return "http://www.metacafe.com/fplayer/" + RegExp.$1 + ".swf";
	    } else if (src.match(/guba.com\/watch\/(\d+)/)){
	    	return "http://www.guba.com/f/root.swf?video_url=http://free.guba.com/uploaditem/" + RegExp.$1 + "/flash.flv&isEmbeddedPlayer=true";
	    } else if (src.match(/nicovideo.jp\/watch\/([^\&]+)/)){
	    	//return "http://nicopon.jp/video/FlowPlayer.swf?config={videoFile:%20'http://nicopon.jp/video/src/" + RegExp.$1 + "',%20initialScale:'fit'}";
	    	return "http://www.nicovideo.jp/thumb/" + RegExp.$1
	    } else if (src.match(/nicovideo.jp\/.*\?v=([^\&]+)/)){
	    	//return "http://nicopon.jp/video/FlowPlayer.swf?config={videoFile:%20'http://nicopon.jp/video/src/" + RegExp.$1 + "',%20initialScale:'fit'}";
	    	return "http://www.nicovideo.jp/thumb/" + RegExp.$1
	    } else {
	    	return false;
	    }
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		//var src = node.href;
		var src = node.rel;
		//if (src.match(/\?url=/)) src = RegExp.rightContext;
		if (ImagePopup.isImage(src)) return;
		if (src.match(/2ch.net/)) return;
		var element = this.getElement(src, false);
		if (element)
			if (SkinPref.getBool("enableVideoPopup", Alt_enableVideoPopup)) this.show(node, src, element);
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source   トリガー元の要素
	 * @param {String}  href     ソース URI
	 * @param {element} obj      埋め込むオブジェクト
	 */
	show: function(source, href, obj) {
		var nodePopup = Popup.add(obj, source);
		if (nodePopup) {
			if (href.match(/nicovideo\.jp/)) {
				nodePopup.container.className = "popupIframe";
				nodePopup.content.style.maxWidth  = obj.width  - 7;
				nodePopup.content.style.maxHeight = obj.height - 7;
			} else
				nodePopup.container.className = "popupVideo";
			Popup.reposition(nodePopup);
		}
	},
	getElement: function(src, is_embed) {
		var href = this.getVideoSource(src, is_embed);
		if (href)
			if (href.match(/nicovideo\.jp/)) {
				var iframe = document.createElement("iframe");
				iframe.width = 340;
				iframe.height = 200;
				iframe.src = href;
				return iframe;
			} else {
				var obj = document.createElement("object");
				obj.setAttribute("type", href.match(/\.divx$/) ? "video/divx" : "application/x-shockwave-flash");
				obj.setAttribute("data", href);
				obj.setAttribute("width",  "340");
				obj.setAttribute("height", "288");
				if (!is_embed) {
					var param = document.createElement("param");
					param.setAttribute("name",  "wmode");
					param.setAttribute("value", "transparent"); // transparent -> opaque にすべき？
					obj.appendChild(param);
				}
				return obj;
			}
	}
};


/**
 * フッタの「おすすめ２ちゃんねる」のポップアップを管理します。
 * @static
 */
var Osusume2chPopup = {
	/** イベントリスナを登録します。*/
	initialise: function() {
		var node = document.getElementById("osusume2ch");

		if (SkinPref.getBool("enableRecomend2chPopupOnClick", Alt_enableRecomend2chPopupOnClick))
			node.addEventListener("click", this.onMouseOver.bind(this), false);
		else
			node.addEventListener("mouseover", this.onMouseOver.bind(this), false);
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		this.show(node);
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source トリガー元の要素
	 */
	show: function(source) {
		var iframe = document.createElement("iframe");
		iframe.width = 500;
		iframe.height = 200;
		iframe.src = "http://" + ThreadDocument.host + "/" + ThreadDocument.boardName + "/i/" + ThreadDocument.threadID + ".html";
		var nodePopup = Popup.add(iframe, source);
		if (nodePopup) {
			nodePopup.container.className = "popupIframe";
			nodePopup.content.style.maxWidth  = iframe.width;
			nodePopup.content.style.maxHeight = iframe.height;
			Popup.reposition(nodePopup);
		}
	}
};

/**
 * フッタの「関連キーワード」のポップアップを管理します。
 * @static
 */
var RelatedKeywordsPopup = {
	/** イベントリスナを登録します。*/
	initialise: function() {
		var node = document.getElementById("relatedKeywords");

		if (SkinPref.getBool("enableRelatedKeywordsPopupOnClick", Alt_enableRelatedKeywordsPopupOnClick))
			node.addEventListener("click", this.onMouseOver.bind(this), false);
		else
			node.addEventListener("mouseover", this.onMouseOver.bind(this), false);
	},
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		this.show(node);
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source トリガー元の要素
	 */
	show: function(source) {
		var iframe = document.createElement("iframe");
		iframe.width = 400;
		iframe.style.height = "1.5em";
		iframe.src = "http://p2.2ch.io/getf.cgi?" + EXACT_URL;
		var nodePopup = Popup.add(iframe, source);
		if (nodePopup) {
			nodePopup.container.className = "popupIframe";
			nodePopup.content.style.maxWidth  = iframe.offsetWidth;
			nodePopup.content.style.maxHeight = iframe.offsetHeight;
			Popup.reposition(nodePopup);
		}
	}
};

/**
 * 2ch スレッド情報のポップアップを管理します。
 * @static
 */
var ThreadInfoPopup = {
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		var node = e.target;
		var src = node.rel;
		if (src.match(/2ch.net/) || src.match(/read\.cgi/)) this.show(node, src);
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source   トリガー元の要素
	 * @param {String}  href     リンク先の URI
	 */
	show: function(source, href) {
		if (!SkinPref.getBool("enableThreadInfoPopup", Alt_enableThreadInfoPopup)) return;

		var cgi = (href.indexOf("read.cgi") != -1 ? true : false);
		var board;
		if (cgi) {
			href.match(/.*test\/read\.cgi\/(.*?)\/.*/);
			board = RegExp.$1;
		} else {
			href.match(/^http:\/\/[^\/]+?\/([^\/]+?)\//);
			board = RegExp.$1;
		}
		var name = boardTable[board];
		name = "板名：" + (name ? name : "不明");

		if (cgi) {
			var title = "";
			var nums = "";
			href.match(/(.+)\/[^\/]*$/);
			var html = ajaxXMLHTTPRequestGet(SERVER_URL + RegExp.$1 + "/1n?lite=true");
			if (html) {
				if (html.match(/<!\-\-__TN\[(.+)\]__\-\->/)) {
					title = RegExp.$1;
					if (html.match(/<!\-\-__RC\[(.+)\]__\-\->/)) {
						nums = RegExp.$1;
					}
				} else {
					title = "";
				}
			} else {
				title = "";
			}

			title = "スレッド名：" + (title ? title : "不明");
			nums = "件数：" + (nums ? nums + " 件" : "不明");
		}

		var content = document.createDocumentFragment();
		var div = document.createElement("div");
		div.className = "popupHeader";

		div.appendChild(document.createTextNode(name));
		if (cgi) {
			var br = document.createElement("br");
			div.appendChild(br.cloneNode(false));
			div.appendChild(document.createTextNode(title));
			div.appendChild(br.cloneNode(false));
			div.appendChild(document.createTextNode(nums));
		}

		content.appendChild(div);
		var nodePopup = Popup.add(content, source);
	}
};

var TemplatePopup = {
	id: null,
	start: null,

	selectRes: function(node) {
		var raw_id   = ResNodes.getIDs(node, true).first.getAttribute("rel");
		var valid_id = ResNodes.getValidIDs(node, true).first;

		if (raw_id) {
			if (!this.id) {
				if (valid_id) {
					this.id = valid_id.getAttribute("rel");
				}
			} else {
				if (!valid_id) return false;
				if (this.id != valid_id.getAttribute("rel")) return false;
			}
		} else {
			var time = new Date(Analyse.getDateFromString(ResNodes.getDates(node, true).first.textContent));
			time = time.getTime();
			if (!this.start) {
				this.start = time;
			} else {
				var minute = SkinPref.getInt("valueTemplateSelectCheckTime", Alt_valueTemplateSelectCheckTime);
				if (this.start + minute * 60 * 1000 < time) return false;
			}
		}
		return true;
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {event} e   イベント
	 */
	show: function(e) {
		var template = document.getElementById("TemplateBody");
		var top = document.getElementById("header").offsetHeight;
		template.style.top = top + window.scrollY;
		var div = document.createElement("div");
		template.appendChild(div);

		var res = SkinPref.getInt("valueTemplateRes", Alt_valueTemplateRes) * 5 + 10;
//		this.initialise();
		var hotkey = SkinPref.getInt("valueTemplateSelectHotkey", Alt_valueTemplateSelectHotkey);
		if (hotkey == e.ctrlKey) {
			ResPopup.show(div, 1, res, null, this.selectRes);
		} else {
			ResPopup.show(div, 1, res);
		}
	}
};

/**
 * 名前のポップアップを管理します。
 * @static
 */
var NamePopup = {
	/** 要素を複製し、いくつかの属性を削除したものを返却します。
	 * @param  {element} src 要素
	 * @return {element} 複製された要素
	 */
	getCloneNode: function(src){
		var dest = src.cloneNode(true);
		if (dest.id) dest.id = "";
		dest.removeAttribute("name");
		dest.style.display = "block";
		var e = dest.getElementsByTagName("*");
		for (var i = 0, n = e.length; i < n; i++){
			if (e[i].id) e[i].id = "";
			e[i].removeAttribute("name");
		}
		return dest;
	},
	/** click イベントを処理します。
	 * @param  {event} e イベント
	 */
	onClick: function(e) {
		var node = e.target;
		this.show(node, node.textContent);
	},
	/** ポップアップを表示します。onMouseOver() から呼ばれます。
	 * @param {element} source      トリガー元の要素
	 */
	show: function(source, name) {
		var rest = (SkinPref.getInt("valuePopupResMax", Alt_valuePopupResMax) + 1) * 5;
		Name.traverse();
		var items = Name.items[name];
		if (!items) return;

		var content = document.createDocumentFragment();
		for (var i = 0, n = items.length; i < n; i++) {
			var dlPopup = this.getCloneNode(Nodes.getRes(items[i]));
			dlPopup.className = "resPopup";
			content.appendChild(dlPopup);
			rest--;
			if (rest <= 0) break;
		}

		var nodePopup = Popup.add(content, source);
	}
};

var OutlinkPopup = {
	/** mouseover イベントを処理します。
	 * @param  {event} e イベント
	 */
	onMouseOver: function(e) {
		ImagePopup.onMouseOver(e);
		UrlPopup.onMouseOver(e);
		VideoPopup.onMouseOver(e);
		ThreadInfoPopup.onMouseOver(e);
	}
};
